#!/bin/bash
HERE=$(pwd)
MY_DIR=$(cd $(dirname $0); pwd)
VERSION=v3
DOWNLOAD_DIR=${HERE}/downloads
PATCHES_DIR=${MY_DIR}/patches

Usage ()
{
	echo "Usage: $0 <command> <target> [<path-to-cross-compiler>]"
	echo "  command set:"
	echo "     all          build everything"
	echo "     clean        clean everything"
	echo "     dist-bin     tar all binaries and data sets in a unique tarball"
	echo "     dist-source  tar all sources in a unique tarball"
	echo "  path-to-cross-compiler:"
	echo "     Path to directory of gcc C/C++ cross-compiler for ${TARGET}"
	echo "     either as argument or into environment variable CROSS_COMPILER_PREFIX"
	echo "     (e.g. /opt/crosstool/gcc-4.4.3-glibc-2.9/${TARGET})"
	echo "     Use crosstool-0.43 or preferably crosstool-ng to get a working cross compiler."
}

function Download
{
	NAME="$1"
	ARCHIVE_NAME="$2"
	ARCHIVE_URL="$3"
	PATCH="$4"

	mkdir -p "${DOWNLOAD_DIR}"

	ARCHIVE="${DOWNLOAD_DIR}/${ARCHIVE_NAME}"
	if test ! -f "${ARCHIVE}"; then
		cd "${DOWNLOAD_DIR}"
		printf "Downloading ${ARCHIVE_URL}"
		success=no
		until [ $success = yes ]; do
			if wget --timeout=60 -t 1 -q  -O "${ARCHIVE_NAME}" "${ARCHIVE_URL}"; then
				success=yes
			else
				printf "."
			fi
		done
		printf "\n"
	fi

	if test ! -d "${TOP_BUILD_DIR}/${NAME}"; then
		mkdir -p "${TOP_BUILD_DIR}"
		cd "${TOP_BUILD_DIR}" || exit
		rm -rf "${NAME}"

		echo "Unpacking ${ARCHIVE_NAME}"
		ext=`echo "${ARCHIVE}" | awk -F . '{print $NF}'`
		case ${ext} in
			bz2)
				tar jxf "${ARCHIVE}" || exit
				;;
			gz)
				tar zxf "${ARCHIVE}" || exit
				;;
			zip)
				unzip -qo "${ARCHIVE}"
				;;
			lzma)
				tar Yxf "${ARCHIVE}" || exit
				;;
		esac

		if test -n "${PATCH}"; then
			echo "Patching ${NAME}"
			cd "${TOP_BUILD_DIR}/${NAME}"
			case "${TARGET}" in
				powerpc*)
					cat "${PATCHES_DIR}/${PATCH}" | sed -e 's/+\tgcc/+\tgcc -mno-altivec/g' | patch -p1 || exit
					;;
				*)
					patch -p1 < "${PATCHES_DIR}/${PATCH}" || exit
					;;
			esac
		fi
	fi
}

CMD="$1"
TARGET="$2"
case "${CMD}" in
	all)
		if [ -z "${CROSS_COMPILER_PREFIX}" ]; then
				if [ -z "$3" ]; then
						echo "FATAL: Please specify path to cross-compiler"
						Usage
						exit 255
				fi
				CROSS_COMPILER_PREFIX="$3"
		fi
		if [ -z "${TARGET}" ]; then
			echo "FATAL: Please specify a target"
			Usage
			exit 255
		fi
		;;
	dist-source)
		;;
	dist-bin)
    if [ -z "${TARGET}" ]; then
      echo "FATAL: Please specify a target"
      Usage
      exit 255
    fi
		;;
	clean)
    if [ -z "${TARGET}" ]; then
      echo "FATAL: Please specify a target"
      Usage
      exit 255
    fi
		;;
	*)
		echo "FATAL: Please specify a command"
		Usage
		exit 255
		;;
esac

TOP_BUILD_DIR=${HERE}/builds/${TARGET}
STATUS=0

case "${CMD}" in
	all)
		Download security security.tar.gz http://www.eecs.umich.edu/mibench/security.tar.gz patch-security
		Download automotive automotive.tar.gz http://www.eecs.umich.edu/mibench/automotive.tar.gz patch-automotive
		Download network network.tar.gz http://www.eecs.umich.edu/mibench/network.tar.gz patch-network
		Download telecomm telecomm.tar.gz http://www.eecs.umich.edu/mibench/telecomm.tar.gz patch-telecomm
		Download consumer consumer.tar.gz http://www.eecs.umich.edu/mibench/consumer.tar.gz
		Download office office.tar.gz http://www.eecs.umich.edu/mibench/office.tar.gz patch-office

		BUILD_DIRS="\
		security/sha \
		security/blowfish \
		security/rijndael \
		security/pgp/src \
		automotive/qsort \
		automotive/susan \
		automotive/basicmath \
		automotive/bitcount \
		network/dijkstra \
		network/patricia \
		telecomm/FFT \
		telecomm/gsm \
		telecomm/CRC32 \
		telecomm/adpcm/src \
		consumer/typeset/lout-3.24 \
		consumer/lame/lame3.70 \
		consumer/jpeg/jpeg-6a \
		office/ispell \
		office/stringsearch \
		office/ghostscript/src \
		office/rsynth \
		office/ispell \
		"

		#Benchmarks i can't get to cross-compile:
		# - consumer/mad/mad-0.14.2b
		# - consumer/tiff-v3.5.4
		# - office/sphinx
		#Benchmarks i can't get to run on powerpc:
		# - office/ghostscript/src (seg fault)
		# - office/rsynth (no output)
		# - office/ispell (dataset is sensitive to endian)

		for BUILD_DIR in ${BUILD_DIRS}; do
			echo "Entering directory ${TOP_BUILD_DIR}/${BUILD_DIR}"
			cd ${TOP_BUILD_DIR}/${BUILD_DIR}
			case ${BUILD_DIR} in
				*ghostscript* | *tiff*)
					;;
				*)
					if [ "$(gcc -v 2>&1 | grep Target | cut -f 2 -d ' ')" != "${TARGET}" ]; then
						if [ -e ${CROSS_COMPILER_PREFIX}/bin/${TARGET}-gcc ]; then
							ln -sf ${CROSS_COMPILER_PREFIX}/bin/${TARGET}-gcc gcc
						fi
						if [ -e ${CROSS_COMPILER_PREFIX}/bin/${TARGET}-ld ]; then
							ln -sf ${CROSS_COMPILER_PREFIX}/bin/${TARGET}-ld ld
						fi
						if [ -e gcc ]; then
							ln -sf gcc cc
						fi
						if [ -e ${CROSS_COMPILER_PREFIX}/bin/${TARGET}-strip ]; then
							ln -sf ${CROSS_COMPILER_PREFIX}/bin/${TARGET}-strip strip
						fi
						if [ -e ${CROSS_COMPILER_PREFIX}/bin/${TARGET}-ar ]; then
							ln -sf ${CROSS_COMPILER_PREFIX}/bin/${TARGET}-ar ar
						fi
						if [ -e ${CROSS_COMPILER_PREFIX}/bin/${TARGET}-ranlib ]; then
							ln -sf ${CROSS_COMPILER_PREFIX}/bin/${TARGET}-ranlib ranlib
						fi
					fi
					;;
			esac
			case ${BUILD_DIR} in
				*pgp*)
					make clean
					;;
				*jpeg*)
					touch jconfig.h
					;;
			esac
			case ${BUILD_DIR} in
				*mad* | *tiff* | *sphinx* | *rsynth)
					unset CONFIGURE_ARGS
					declare -a CONFIGURE_ARGS
					case ${BUILD_DIR} in
						*tiff*)
							CONFIGURE_ARGS[0]="CC=${CROSS_COMPILER_PREFIX}/bin/${TARGET}-gcc"
							CONFIGURE_ARGS[1]="--target=${TARGET}"
							CONFIGURE_ARGS[2]="--noninteractive"
							;;
						*rsynth)
							CONFIGURE_ARGS[0]="CC=${CROSS_COMPILER_PREFIX}/bin/${TARGET}-gcc"
							CONFIGURE_ARGS[1]="--host=${TARGET}"
							;;
						*)
							;;
					esac
					echo "./configure ${CONFIGURE_ARGS[@]}"
					PATH=$(pwd):${PATH} ./configure "${CONFIGURE_ARGS[@]}"
					STATUS=$?
					;;
				*)
					;;
			esac
			case ${BUILD_DIR} in
				*sha | *blowfish | *rijndael | *pgp* | *susan | *basicmath | *qsort | *bitcount | \
				*dijkstra | *patricia | *FFT | *gsm | *CRC32 | *adpcm* | *typeset* | *lame* | \
				*jpeg* | *ghostscript* | *ispell | *stringsearch | *tiff* | *sphinx* | *rsynth)
					unset BUILD_ARGS
					declare -a BUILD_ARGS
					case ${BUILD_DIR} in
						*sha)
							BUILD_ARGS[0]="CFLAGS=-static -O3 -Wall -g"
							case ${TARGET} in
								powerpc*)
									BUILD_ARGS[0]="${BUILD_ARGS[0]} -mno-altivec"
									;;
							esac
							BUILD_ARGS[1]="sha"
							;;
						*blowfish)
							BUILD_ARGS[0]="CFLAGS=-static -O3 -g"
							case ${TARGET} in
								powerpc*)
									BUILD_ARGS[0]="${BUILD_ARGS[0]} -mno-altivec"
									;;
							esac
							;;
						*rijndael)
							BUILD_ARGS[0]="rijndael"
							;;
						*susan)
							BUILD_ARGS[0]="susan"
							;;
						*bitcount)
							BUILD_ARGS[0]="bitcnts"
							;;
						*patricia)
							BUILD_ARGS[0]="patricia"
							;;
						*FFT)
							BUILD_ARGS[0]="fft"
							BUILD_ARGS[1]="CFLAGS=-static -O3 -g"
							case ${TARGET} in
								powerpc*)
									BUILD_ARGS[1]="${BUILD_ARGS[1]} -mno-altivec"
									;;
							esac
							;;
						*CRC32)
							BUILD_ARGS[0]="crc"
							;;
						*typeset*)
							BUILD_ARGS[0]="lout"
							BUILD_ARGS[1]="LD=gcc -static"
							BUILD_ARGS[2]="COPTS=-ansi -pedantic -Wall -O3 -g"
							case ${TARGET} in
								powerpc*)
									BUILD_ARGS[2]="${BUILD_ARGS[2]} -mno-altivec"
									;;
							esac
							;;
						*pgp*)
							BUILD_ARGS[0]="all"
							BUILD_ARGS[1]="OBJS_EXT="
							BUILD_ARGS[2]="LDFLAGS=-static -g"
							BUILD_ARGS[3]="CFLAGS=-O2 -g -DUNIX -DLINUX -DIDEA32 -DPORTABLE -D__STRICT_ANSI__"
							case "${TARGET}" in
								powerpc*)
									BUILD_ARGS[3]="${BUILD_ARGS[3]} -DHIGHFIRST -mno-altivec"
									;;
							esac
							;;
						*lame*)
							BUILD_ARGS[0]="lame"
							BUILD_ARGS[1]="BRHIST_SWITCH="
							BUILD_ARGS[2]="LIBTERMCAP="
							BUILD_ARGS[3]="CC=gcc -static"
							BUILD_ARGS[4]="CC_OPTS=-O3 -fomit-frame-pointer -funroll-loops -ffast-math  -finline-functions -Wall -g"
							case "${TARGET}" in
								i*86*)
									BUILD_ARGS[4]="${BUILD_ARGS[4]} -U__i386__"
									;;
								powerpc*)
									BUILD_ARGS[4]="${BUILD_ARGS[4]} -mno-altivec"
									;;
							esac
							;;
						*ghostscript*)
							BUILD_ARGS[0]="all"
							if [ -e "${CROSS_COMPILER_PREFIX}/${TARGET}/sysroot/usr/include" ]; then
								BUILD_ARGS[1]="INCLUDE=${CROSS_COMPILER_PREFIX}/${TARGET}/sysroot/usr/include"
							elif [ -e "${CROSS_COMPILER_PREFIX}/${TARGET}/sys-root/usr/include" ]; then
								BUILD_ARGS[1]="INCLUDE=${CROSS_COMPILER_PREFIX}/${TARGET}/sys-root/usr/include"
							else
								echo "WARNING: don't know where are the cross-compiler includes"
								BUILD_ARGS[1]="INCLUDE=${CROSS_COMPILER_PREFIX}/include"
							fi
							BUILD_ARGS[2]="CCAUX=$(which gcc)"
							BUILD_ARGS[3]="CC=${CROSS_COMPILER_PREFIX}/bin/${TARGET}-gcc"
							BUILD_ARGS[4]="LDFLAGS=-static"
							BUILD_ARGS[5]="CFLAGS=-O3 -g -D_POSIX_C_SOURCE=1"
							case ${TARGET} in
								powerpc*)
									BUILD_ARGS[5]="${BUILD_ARGS[5]} -mno-altivec"
									;;
							esac
							;;
						*ispell)
							BUILD_ARGS[0]="ispell"
							;;
						*tiff*)
							BUILD_ARGS[0]="MACHLIBS=-lm"
							;;
						*rsynth)
							BUILD_ARGS[0]="CC=${CROSS_COMPILER_PREFIX}/bin/${TARGET}-gcc"
							BUILD_ARGS[1]="CFLAGS=-g -O3"
							case ${TARGET} in
								powerpc*)
									BUILD_ARGS[1]="${BUILD_ARGS[1]} -mno-altivec"
									;;
							esac
							BUILD_ARGS[2]="LDLIBS=-lm"
							BUILD_ARGS[3]="LDFLAGS=-g -static"
							;;
						*adpcm*)
							BUILD_ARGS[0]="CFLAGS=-static -O3 -g"
							case ${TARGET} in
								powerpc*)
									BUILD_ARGS[0]="${BUILD_ARGS[0]} -mno-altivec"
									;;
							esac
							;;
						*gsm)
							BUILD_ARGS[0]="CC=gcc -ansi -pedantic -g -static"
							case ${TARGET} in
								powerpc*)
									BUILD_ARGS[0]="${BUILD_ARGS[0]} -mno-altivec"
									;;
							esac
							;;
						*jpeg*)
							BUILD_ARGS[0]="CFLAGS= -O3 -I\$(srcdir) -g"
							case ${TARGET} in
								powerpc*)
									BUILD_ARGS[0]="${BUILD_ARGS[0]} -mno-altivec"
									;;
							esac
							BUILD_ARGS[1]="LDFLAGS=-static -g"
							;;
						*)
							BUILD_ARGS[0]="all"
							;;
					esac
					echo "make ${BUILD_ARGS[@]}"
					PATH=$(pwd):${PATH} make "${BUILD_ARGS[@]}"
					STATUS=$?
					;;
				*)
					echo "Don't know how to build MiBench in ${BUILD_DIR}"
					STATUS=1
					;;
			esac
			case ${BUILD_DIR} in
				*pgp*)
					sed -e '/^$/d' -e '3,$ s/^\(.*\)$/\$\{SIMULATE\} \1/' "${TOP_BUILD_DIR}/${BUILD_DIR}/../runme.sh" > "${TOP_BUILD_DIR}/${BUILD_DIR}/../simme.sh"
					chmod +x "${TOP_BUILD_DIR}/${BUILD_DIR}/../simme.sh"
					;;
				*typeset* | *lame* | *jpeg* | *adpcm* | *ghostscript*)
					sed -e '/^$/d' -e '2,$ s/^\(.*\)$/\$\{SIMULATE\} \1/' "${TOP_BUILD_DIR}/${BUILD_DIR}/../runme_small.sh" > "${TOP_BUILD_DIR}/${BUILD_DIR}/../simme_small.sh"
					sed -e '/^$/d' -e '2,$ s/^\(.*\)$/\$\{SIMULATE\} \1/' "${TOP_BUILD_DIR}/${BUILD_DIR}/../runme_large.sh" > "${TOP_BUILD_DIR}/${BUILD_DIR}/../simme_large.sh"
					chmod +x "${TOP_BUILD_DIR}/${BUILD_DIR}/../simme_small.sh"
					chmod +x "${TOP_BUILD_DIR}/${BUILD_DIR}/../simme_large.sh"
					;;
				*)
					sed -e '/^$/d' -e '2,$ s/^\(.*\)$/\$\{SIMULATE\} \1/' "${TOP_BUILD_DIR}/${BUILD_DIR}/runme_small.sh" > "${TOP_BUILD_DIR}/${BUILD_DIR}/simme_small.sh"
					sed -e '/^$/d' -e '2,$ s/^\(.*\)$/\$\{SIMULATE\} \1/' "${TOP_BUILD_DIR}/${BUILD_DIR}/runme_large.sh" > "${TOP_BUILD_DIR}/${BUILD_DIR}/simme_large.sh"
					chmod +x "${TOP_BUILD_DIR}/${BUILD_DIR}/simme_small.sh"
					chmod +x "${TOP_BUILD_DIR}/${BUILD_DIR}/simme_large.sh"
					;;
			esac
			rm -f gcc
			rm -f ld
			rm -f cc
			rm -f strip
			rm -f ar
			rm -f ranlib
			echo "Leaving directory ${TOP_BUILD_DIR}/${BUILD_DIR}"
			cd ${MY_DIR}
			if [ ${STATUS} -ne 0 ]; then
				break
			fi
		done
		;;
	dist-source)
		FILE_LIST="\
		downloads/automotive.tar.gz \
		downloads/consumer.tar.gz \
		downloads/network.tar.gz \
		downloads/office.tar.gz \
		downloads/security.tar.gz \
		downloads/telecomm.tar.gz \
		patches/patch-automotive \
		patches/patch-network \
		patches/patch-office \
		patches/patch-security \
		patches/patch-telecomm \
		$(basename $0) \
		auto.sh \
		"

		PKG_NAME="mibench-v1-source-${VERSION}"

		rm -rf ${HERE}/${PKG_NAME}
		rm -f ${HERE}/${PKG_NAME}.tar.gz
		mkdir -p ${HERE}/${PKG_NAME}

		for FILE in ${FILE_LIST}; do
			mkdir -p ${HERE}/${PKG_NAME}/$(dirname ${FILE})
			cp ${MY_DIR}/${FILE} ${HERE}/${PKG_NAME}/${FILE} || exit -1
		done

		cat << EOF > "${HERE}/${PKG_NAME}/README"
This package contains:
- the sources of MiBench version 1 (http://www.eecs.umich.edu/mibench)
- some patches and a building script to cross-compile them (with a GCC cross-compiler)
EOF
		cd ${HERE}
		tar zcvf ${HERE}/${PKG_NAME}.tar.gz ${PKG_NAME}
		rm -rf ${HERE}/${PKG_NAME}
		;;
	dist-bin)
		FILE_LIST="\
		security/sha/runme_small.sh \
		security/sha/simme_small.sh \
		security/sha/sha \
		security/sha/input_small.asc \
		security/sha/runme_large.sh \
		security/sha/simme_large.sh \
		security/sha/input_large.asc \
		security/blowfish/runme_small.sh \
		security/blowfish/simme_small.sh \
		security/blowfish/bf \
		security/blowfish/input_small.asc \
		security/blowfish/runme_large.sh \
		security/blowfish/simme_large.sh \
		security/blowfish/input_large.asc \
		security/rijndael/runme_small.sh \
		security/rijndael/simme_small.sh \
		security/rijndael/rijndael \
		security/rijndael/input_small.asc \
		security/rijndael/runme_large.sh \
		security/rijndael/simme_large.sh \
		security/rijndael/input_large.asc \
		security/pgp/runme.sh \
		security/pgp/simme.sh \
		security/pgp/src/pgp \
		security/pgp/tests/config.txt \
		security/pgp/tests/PGPMacBinaryMappings.txt \
		security/pgp/tests/pubring.bak \
		security/pgp/tests/randseed.bin \
		security/pgp/tests/secring.pgp \
		security/pgp/tests/testin.txt \
		security/pgp/tests/testout.txt.asc \
		security/pgp/tests/PGPgroup.pgr \
		security/pgp/tests/PGPsdkPreferences \
		security/pgp/tests/pubring.pgp \
		security/pgp/tests/randseed.rnd \
		security/pgp/tests/secring.bak \
		security/pgp/tests/secring.skr \
		automotive/qsort/runme_small.sh \
		automotive/qsort/simme_small.sh \
		automotive/qsort/qsort_small \
		automotive/qsort/input_small.dat \
		automotive/qsort/runme_large.sh \
		automotive/qsort/simme_large.sh \
		automotive/qsort/qsort_large \
		automotive/qsort/input_large.dat \
		automotive/susan/runme_small.sh \
		automotive/susan/simme_small.sh \
		automotive/susan/susan \
		automotive/susan/input_small.pgm \
		automotive/susan/runme_large.sh \
		automotive/susan/simme_large.sh \
		automotive/susan/input_large.pgm \
		automotive/basicmath/runme_small.sh \
		automotive/basicmath/simme_small.sh \
		automotive/basicmath/basicmath_small \
		automotive/basicmath/runme_large.sh \
		automotive/basicmath/simme_large.sh \
		automotive/basicmath/basicmath_large \
		automotive/bitcount/runme_small.sh \
		automotive/bitcount/simme_small.sh \
		automotive/bitcount/bitcnts \
		automotive/bitcount/runme_large.sh \
		automotive/bitcount/simme_large.sh \
		network/dijkstra/runme_small.sh \
		network/dijkstra/simme_small.sh \
		network/dijkstra/dijkstra_small \
		network/dijkstra/input.dat \
		network/dijkstra/runme_large.sh \
		network/dijkstra/simme_large.sh \
		network/dijkstra/dijkstra_large \
		network/patricia/runme_small.sh \
		network/patricia/simme_small.sh \
		network/patricia/patricia \
		network/patricia/small.udp \
		network/patricia/runme_large.sh \
		network/patricia/simme_large.sh \
		network/patricia/large.udp \
		telecomm/FFT/runme_small.sh \
		telecomm/FFT/simme_small.sh \
		telecomm/FFT/fft \
		telecomm/FFT/runme_large.sh \
		telecomm/FFT/simme_large.sh \
		telecomm/gsm/runme_small.sh \
		telecomm/gsm/simme_small.sh \
		telecomm/gsm/bin/toast \
		telecomm/gsm/bin/untoast \
		telecomm/gsm/data/small.au \
		telecomm/gsm/data/small.au.run.gsm \
		telecomm/gsm/runme_large.sh \
		telecomm/gsm/simme_large.sh \
		telecomm/gsm/data/large.au \
		telecomm/gsm/data/large.au.run.gsm \
		telecomm/CRC32/runme_small.sh \
		telecomm/CRC32/simme_small.sh \
		telecomm/CRC32/crc \
		telecomm/CRC32/runme_large.sh \
		telecomm/CRC32/simme_large.sh \
		telecomm/adpcm/runme_small.sh \
		telecomm/adpcm/simme_small.sh \
		telecomm/adpcm/bin/rawcaudio \
		telecomm/adpcm/bin/rawdaudio \
		telecomm/adpcm/data/small.pcm \
		telecomm/adpcm/data/small.adpcm \
		telecomm/adpcm/runme_large.sh \
		telecomm/adpcm/simme_large.sh \
		telecomm/adpcm/data/large.pcm \
		telecomm/adpcm/data/large.adpcm \
		consumer/typeset/runme_small.sh \
		consumer/typeset/simme_small.sh \
		consumer/typeset/small.lout \
		consumer/typeset/lout-3.24/lout \
		consumer/typeset/runme_large.sh \
		consumer/typeset/simme_large.sh \
		consumer/typeset/large.lout \
		consumer/typeset/lout-3.24/include/tbl \
		consumer/typeset/lout-3.24/include/eiffel \
		consumer/typeset/lout-3.24/include/bsf \
		consumer/typeset/lout-3.24/include/modulaf \
		consumer/typeset/lout-3.24/include/pasf \
		consumer/typeset/lout-3.24/include/slides \
		consumer/typeset/lout-3.24/include/bookf \
		consumer/typeset/lout-3.24/include/graphf \
		consumer/typeset/lout-3.24/include/report \
		consumer/typeset/lout-3.24/include/pythonf \
		consumer/typeset/lout-3.24/include/tabf.lpg \
		consumer/typeset/lout-3.24/include/figf \
		consumer/typeset/lout-3.24/include/pod \
		consumer/typeset/lout-3.24/include/diagf.lpg \
		consumer/typeset/lout-3.24/include/figf.lpg \
		consumer/typeset/lout-3.24/include/mydefs \
		consumer/typeset/lout-3.24/include/russian \
		consumer/typeset/lout-3.24/include/cprint \
		consumer/typeset/lout-3.24/include/diagf \
		consumer/typeset/lout-3.24/include/eq \
		consumer/typeset/lout-3.24/include/reportf \
		consumer/typeset/lout-3.24/include/python \
		consumer/typeset/lout-3.24/include/cprintf \
		consumer/typeset/lout-3.24/include/modula \
		consumer/typeset/lout-3.24/include/tblf.lpg \
		consumer/typeset/lout-3.24/include/langdefs \
		consumer/typeset/lout-3.24/include/book \
		consumer/typeset/lout-3.24/include/diagf.etc \
		consumer/typeset/lout-3.24/include/tabf \
		consumer/typeset/lout-3.24/include/perlf \
		consumer/typeset/lout-3.24/include/podf \
		consumer/typeset/lout-3.24/include/perl \
		consumer/typeset/lout-3.24/include/ts \
		consumer/typeset/lout-3.24/include/eqf \
		consumer/typeset/lout-3.24/include/slidesf \
		consumer/typeset/lout-3.24/include/russian.fd \
		consumer/typeset/lout-3.24/include/picture \
		consumer/typeset/lout-3.24/include/doc \
		consumer/typeset/lout-3.24/include/docf \
		consumer/typeset/lout-3.24/include/bsf.lpg \
		consumer/typeset/lout-3.24/include/fig \
		consumer/typeset/lout-3.24/include/tblf \
		consumer/typeset/lout-3.24/include/blue \
		consumer/typeset/lout-3.24/include/README \
		consumer/typeset/lout-3.24/include/eiffelf \
		consumer/typeset/lout-3.24/include/graphf.lpg \
		consumer/typeset/lout-3.24/include/tab \
		consumer/typeset/lout-3.24/include/picturef \
		consumer/typeset/lout-3.24/include/init \
		consumer/typeset/lout-3.24/include/dsf \
		consumer/typeset/lout-3.24/include/pas \
		consumer/typeset/lout-3.24/include/bluef \
		consumer/typeset/lout-3.24/include/latin2 \
		consumer/typeset/lout-3.24/include/graphf.etc \
		consumer/typeset/lout-3.24/include/graph \
		consumer/typeset/lout-3.24/include/diag \
		consumer/typeset/lout-3.24/data/loutrefs.li \
		consumer/typeset/lout-3.24/data/fontdefs.li \
		consumer/typeset/lout-3.24/data/latin2.ld \
		consumer/typeset/lout-3.24/data/loutrefs.ld \
		consumer/typeset/lout-3.24/data/fcvt \
		consumer/typeset/lout-3.24/data/refstyle.ld \
		consumer/typeset/lout-3.24/data/standard.li \
		consumer/typeset/lout-3.24/data/fontdefs.ld \
		consumer/typeset/lout-3.24/data/standard.ld \
		consumer/typeset/lout-3.24/data/README \
		consumer/typeset/lout-3.24/data/refstyle.li \
		consumer/typeset/lout-3.24/data/oldrefs.ld \
		consumer/typeset/lout-3.24/data/fcvt.awk \
		consumer/typeset/lout-3.24/font/He-NrBdO+ \
		consumer/typeset/lout-3.24/font/ZC-Rm+ \
		consumer/typeset/lout-3.24/font/He-NrO+ \
		consumer/typeset/lout-3.24/font/Symbol \
		consumer/typeset/lout-3.24/font/Bk-MdIt \
		consumer/typeset/lout-3.24/font/He-BdO \
		consumer/typeset/lout-3.24/font/Cr-O+ \
		consumer/typeset/lout-3.24/font/Cr-O \
		consumer/typeset/lout-3.24/font/Pa-BdF \
		consumer/typeset/lout-3.24/font/Ti-BdSC+ \
		consumer/typeset/lout-3.24/font/AG-BdO+ \
		consumer/typeset/lout-3.24/font/Ti-RmSC \
		consumer/typeset/lout-3.24/font/He-Bl+ \
		consumer/typeset/lout-3.24/font/He-CnBd \
		consumer/typeset/lout-3.24/font/He-CnBlO \
		consumer/typeset/lout-3.24/font/AG-CnMd \
		consumer/typeset/lout-3.24/font/Bk-Dm+ \
		consumer/typeset/lout-3.24/font/Ti-BdItF \
		consumer/typeset/lout-3.24/font/Ti-Sm+ \
		consumer/typeset/lout-3.24/font/He-NrBd+ \
		consumer/typeset/lout-3.24/font/Ti-It+ \
		consumer/typeset/lout-3.24/font/Bk-BdIt+ \
		consumer/typeset/lout-3.24/font/He-Bl \
		consumer/typeset/lout-3.24/font/He-O+ \
		consumer/typeset/lout-3.24/font/Pa-Bd+ \
		consumer/typeset/lout-3.24/font/He-ExtC \
		consumer/typeset/lout-3.24/font/He-O \
		consumer/typeset/lout-3.24/font/AG-CnDm+ \
		consumer/typeset/lout-3.24/font/Bk-Dm \
		consumer/typeset/lout-3.24/font/ZC-Lt+ \
		consumer/typeset/lout-3.24/font/He-Bd \
		consumer/typeset/lout-3.24/font/He-CnLiO \
		consumer/typeset/lout-3.24/font/Ti-Rm \
		consumer/typeset/lout-3.24/font/He-Cm \
		consumer/typeset/lout-3.24/font/AG-Dm+ \
		consumer/typeset/lout-3.24/font/ZC-Bd+ \
		consumer/typeset/lout-3.24/font/Pa-Rm+ \
		consumer/typeset/lout-3.24/font/AG-CnDm \
		consumer/typeset/lout-3.24/font/He-Lt \
		consumer/typeset/lout-3.24/font/AG-DmO+ \
		consumer/typeset/lout-3.24/font/NCS-It+ \
		consumer/typeset/lout-3.24/font/Bk-MdIt+ \
		consumer/typeset/lout-3.24/font/AG-ExLt \
		consumer/typeset/lout-3.24/font/He-CnO+ \
		consumer/typeset/lout-3.24/font/He-CnBl \
		consumer/typeset/lout-3.24/font/He-Cn+ \
		consumer/typeset/lout-3.24/font/He-CnLiO+ \
		consumer/typeset/lout-3.24/font/NCS-Bd \
		consumer/typeset/lout-3.24/font/ZC-Bd \
		consumer/typeset/lout-3.24/font/Ti-ExBd \
		consumer/typeset/lout-3.24/font/ZC-LtIt+ \
		consumer/typeset/lout-3.24/font/He-CnBdO \
		consumer/typeset/lout-3.24/font/He-CnBdO+ \
		consumer/typeset/lout-3.24/font/NCS-It \
		consumer/typeset/lout-3.24/font/AG-Bk \
		consumer/typeset/lout-3.24/font/He-NrBd \
		consumer/typeset/lout-3.24/font/He-CnBl+ \
		consumer/typeset/lout-3.24/font/Ti-Bd+ \
		consumer/typeset/lout-3.24/font/Pa-Rm \
		consumer/typeset/lout-3.24/font/Ti-Sm \
		consumer/typeset/lout-3.24/font/He-Cm+ \
		consumer/typeset/lout-3.24/font/ZD \
		consumer/typeset/lout-3.24/font/He-CnO \
		consumer/typeset/lout-3.24/font/He-Cn \
		consumer/typeset/lout-3.24/font/Ti-RmSC+ \
		consumer/typeset/lout-3.24/font/He-Nr+ \
		consumer/typeset/lout-3.24/font/Bk-DmIt+ \
		consumer/typeset/lout-3.24/font/AG-Dm \
		consumer/typeset/lout-3.24/font/Pa-BdF+ \
		consumer/typeset/lout-3.24/font/Ti-BdIt+ \
		consumer/typeset/lout-3.24/font/Pa-It \
		consumer/typeset/lout-3.24/font/He-CnLi \
		consumer/typeset/lout-3.24/font/ZC-Dm+ \
		consumer/typeset/lout-3.24/font/Ti-It \
		consumer/typeset/lout-3.24/font/Ti-ItF+ \
		consumer/typeset/lout-3.24/font/AG-BkO \
		consumer/typeset/lout-3.24/font/AG-CnBd+ \
		consumer/typeset/lout-3.24/font/Ti-BdSC \
		consumer/typeset/lout-3.24/font/AG-MdO+ \
		consumer/typeset/lout-3.24/font/ZC-LtIt \
		consumer/typeset/lout-3.24/font/Cr-Bd+ \
		consumer/typeset/lout-3.24/font/NCS-Rm+ \
		consumer/typeset/lout-3.24/font/Pa-BdIt \
		consumer/typeset/lout-3.24/font/ZC-It \
		consumer/typeset/lout-3.24/font/Ti-ExBd+ \
		consumer/typeset/lout-3.24/font/00README \
		consumer/typeset/lout-3.24/font/Cr+ \
		consumer/typeset/lout-3.24/font/Ti-BdItF+ \
		consumer/typeset/lout-3.24/font/He-UlCm \
		consumer/typeset/lout-3.24/font/He \
		consumer/typeset/lout-3.24/font/AG-ExLtO \
		consumer/typeset/lout-3.24/font/He-ExtC+ \
		consumer/typeset/lout-3.24/font/He-CnBlO+ \
		consumer/typeset/lout-3.24/font/AG-BdO \
		consumer/typeset/lout-3.24/font/ZC-Lt \
		consumer/typeset/lout-3.24/font/AG-Bk+ \
		consumer/typeset/lout-3.24/font/Pa-BdItF+ \
		consumer/typeset/lout-3.24/font/ZC-Rm \
		consumer/typeset/lout-3.24/font/AG-Bd \
		consumer/typeset/lout-3.24/font/Ti-SmIt \
		consumer/typeset/lout-3.24/font/Bk-Bd \
		consumer/typeset/lout-3.24/font/AG-CnBk \
		consumer/typeset/lout-3.24/font/AG-DmO \
		consumer/typeset/lout-3.24/font/He-Nr \
		consumer/typeset/lout-3.24/font/AG-Md+ \
		consumer/typeset/lout-3.24/font/AG-ExLt+ \
		consumer/typeset/lout-3.24/font/He-LtO \
		consumer/typeset/lout-3.24/font/Cr-BdO \
		consumer/typeset/lout-3.24/font/Pa-It+ \
		consumer/typeset/lout-3.24/font/NCS-Rm \
		consumer/typeset/lout-3.24/font/AG-MdO \
		consumer/typeset/lout-3.24/font/Bk-Lt \
		consumer/typeset/lout-3.24/font/He-Lt+ \
		consumer/typeset/lout-3.24/font/ZC-MdIt \
		consumer/typeset/lout-3.24/font/Cr-Bd \
		consumer/typeset/lout-3.24/font/Bk-BdIt \
		consumer/typeset/lout-3.24/font/Bk-Bd+ \
		consumer/typeset/lout-3.24/font/ZC-MdIt+ \
		consumer/typeset/lout-3.24/font/Pa-Bd \
		consumer/typeset/lout-3.24/font/ZC-Dm \
		consumer/typeset/lout-3.24/font/Cr-BdO+ \
		consumer/typeset/lout-3.24/font/Pa-SC+ \
		consumer/typeset/lout-3.24/font/Ti-BdIt \
		consumer/typeset/lout-3.24/font/Bk-Md \
		consumer/typeset/lout-3.24/font/He-BlO \
		consumer/typeset/lout-3.24/font/Bk-LtIt+ \
		consumer/typeset/lout-3.24/font/Ti-Bd \
		consumer/typeset/lout-3.24/font/AG-ExLtO+ \
		consumer/typeset/lout-3.24/font/Pa-BdItF \
		consumer/typeset/lout-3.24/font/He-Bd+ \
		consumer/typeset/lout-3.24/font/AG-BkO+ \
		consumer/typeset/lout-3.24/font/ZC-It+ \
		consumer/typeset/lout-3.24/font/He-CnLi+ \
		consumer/typeset/lout-3.24/font/He-UlCm+ \
		consumer/typeset/lout-3.24/font/Bk-LtIt \
		consumer/typeset/lout-3.24/font/Bk-Lt+ \
		consumer/typeset/lout-3.24/font/Ti-SmIt+ \
		consumer/typeset/lout-3.24/font/He-NrO \
		consumer/typeset/lout-3.24/font/He-BdO+ \
		consumer/typeset/lout-3.24/font/Cr \
		consumer/typeset/lout-3.24/font/Bk-Md+ \
		consumer/typeset/lout-3.24/font/He-LtO+ \
		consumer/typeset/lout-3.24/font/Pa-ItF+ \
		consumer/typeset/lout-3.24/font/AG-CnMd+ \
		consumer/typeset/lout-3.24/font/He+ \
		consumer/typeset/lout-3.24/font/AG-CnBk+ \
		consumer/typeset/lout-3.24/font/Pa-ItF \
		consumer/typeset/lout-3.24/font/AG-CnBd \
		consumer/typeset/lout-3.24/font/Ti-Rm+ \
		consumer/typeset/lout-3.24/font/NCS-BdIt \
		consumer/typeset/lout-3.24/font/Bk-DmIt \
		consumer/typeset/lout-3.24/font/He-BlO+ \
		consumer/typeset/lout-3.24/font/AG-Bd+ \
		consumer/typeset/lout-3.24/font/He-NrBdO \
		consumer/typeset/lout-3.24/font/Pa-SC \
		consumer/typeset/lout-3.24/font/NCS-BdIt+ \
		consumer/typeset/lout-3.24/font/Ti-ItF \
		consumer/typeset/lout-3.24/font/AG-Md \
		consumer/typeset/lout-3.24/font/Pa-BdIt+ \
		consumer/typeset/lout-3.24/font/NCS-Bd+ \
		consumer/typeset/lout-3.24/font/He-CnBd+ \
		consumer/typeset/lout-3.24/maps/null.LCM \
		consumer/typeset/lout-3.24/maps/Std.LCM \
		consumer/typeset/lout-3.24/maps/Symb.LCM \
		consumer/typeset/lout-3.24/maps/Latin1.LCM \
		consumer/typeset/lout-3.24/maps/Ding.LCM \
		consumer/typeset/lout-3.24/maps/LtLatin1.LCM \
		consumer/typeset/lout-3.24/maps/00README \
		consumer/typeset/lout-3.24/maps/LtLatin2.LCM \
		consumer/typeset/lout-3.24/maps/koi8r.LCM \
		consumer/typeset/lout-3.24/hyph/finnish.lh \
		consumer/typeset/lout-3.24/hyph/norweg.lh \
		consumer/typeset/lout-3.24/hyph/slovenia.lh \
		consumer/typeset/lout-3.24/hyph/danish.lh \
		consumer/typeset/lout-3.24/hyph/spanish.lh \
		consumer/typeset/lout-3.24/hyph/dutch.lh \
		consumer/typeset/lout-3.24/hyph/english.lp \
		consumer/typeset/lout-3.24/hyph/italian.lh \
		consumer/typeset/lout-3.24/hyph/czech.lh \
		consumer/typeset/lout-3.24/hyph/german.lh \
		consumer/typeset/lout-3.24/hyph/swedish.lh \
		consumer/typeset/lout-3.24/hyph/portugal.lh \
		consumer/typeset/lout-3.24/hyph/polish.lh \
		consumer/typeset/lout-3.24/hyph/README \
		consumer/typeset/lout-3.24/hyph/engluk.lh \
		consumer/typeset/lout-3.24/hyph/russian.lh \
		consumer/typeset/lout-3.24/hyph/english.lh \
		consumer/typeset/lout-3.24/hyph/french.lh \
		consumer/lame/runme_small.sh \
		consumer/lame/simme_small.sh \
		consumer/lame/lame3.70/lame \
		consumer/lame/runme_large.sh \
		consumer/lame/simme_large.sh \
		consumer/lame/small.wav \
		consumer/lame/large.wav \
		consumer/jpeg/runme_small.sh \
		consumer/jpeg/simme_small.sh \
		consumer/jpeg/jpeg-6a/cjpeg \
		consumer/jpeg/jpeg-6a/djpeg \
		consumer/jpeg/input_small.ppm \
		consumer/jpeg/input_small.jpg \
		consumer/jpeg/runme_large.sh \
		consumer/jpeg/simme_large.sh \
		consumer/jpeg/input_large.ppm \
		consumer/jpeg/input_large.jpg \
		office/ghostscript/runme_small.sh \
		office/ghostscript/simme_small.sh \
		office/ghostscript/src/gs \
		office/ghostscript/data/small.ps \
		office/ghostscript/runme_large.sh \
		office/ghostscript/simme_large.sh \
		office/ghostscript/data/large.ps \
		office/ispell/runme_small.sh \
		office/ispell/simme_small.sh \
		office/ispell/ispell \
		office/ispell/tests/americanmed+.hash \
		office/ispell/tests/small.txt \
		office/ispell/runme_large.sh \
		office/ispell/simme_large.sh \
		office/ispell/tests/large.txt \
		office/stringsearch/runme_small.sh \
		office/stringsearch/simme_small.sh \
		office/stringsearch/search_small \
		office/stringsearch/runme_large.sh \
		office/stringsearch/simme_large.sh \
		office/stringsearch/search_large \
		office/rsynth/runme_small.sh \
		office/rsynth/simme_small.sh \
		office/rsynth/say \
		office/rsynth/smallinput.txt \
		office/rsynth/runme_large.sh \
		office/rsynth/simme_large.sh \
		office/rsynth/largeinput.txt \
		"

		PKG_NAME="mibench-v1-bin-${TARGET}-${VERSION}"

		rm -rf ${HERE}/${PKG_NAME}
		rm -f ${HERE}/${PKG_NAME}.tar.gz
		mkdir -p ${HERE}/${PKG_NAME}

		for FILE in ${FILE_LIST}; do
			mkdir -p ${HERE}/${PKG_NAME}/builds/${TARGET}/$(dirname ${FILE})
			cp ${MY_DIR}/builds/${TARGET}/${FILE} ${HERE}/${PKG_NAME}/builds/${TARGET}/${FILE} || exit -1
		done

		cp ${MY_DIR}/auto.sh ${HERE}/${PKG_NAME}/auto.sh
		cat << EOF > "${HERE}/${PKG_NAME}/README"
This package contains:
- prebuilt binaries of MiBench version 1 (http://www.eecs.umich.edu/mibench) for target ${TARGET}
- the original MiBench version 1 datasets and scripts to run each benchmark on the native target or a simulator
- a script (auto.sh) to automate runs of benchmarks on the native target or a simulator, and comparisons between runs

Note: consumer/mad, consumer/tiff-v3.5.4, consumer/mad/mad-0.14.2b and office/sphinx are not included because we were not able to build them.
EOF
		cd ${HERE}
		tar zcvf ${HERE}/${PKG_NAME}.tar.gz ${PKG_NAME}
		rm -rf ${HERE}/${PKG_NAME}
		;;
	clean)
		rm -rf ${TOP_BUILD_DIR}
		;;
esac

cd ${HERE}


exit ${STATUS}
